(*

  SPINEL - knihovna formtu S97 ("a") / binrn formt

  Vydavatel: Papouch s.r.o. - http://spinel.papouch.com/

  Popis komunikanho protokolu SPINEL, Formt 97

  Dotaz:
   PRE FRM NUM NUM ADR SIG INST DATA SUMA CR
  Odpov:
   PRE FRM NUM NUM ADR SIG ACK DATA SUMA CR

   PRE  - Prefix, ($2A / "*" - znak hvzdika)
   FRM  - slo formtu 97 ($61 / "a"),
   NUM  - Poet bajt instrukce od nsledujcho bajtu do konce rmce.
          viz obecn popis.
   ADR  - Adresa modulu, ktermu je posln dotaz nebo kter posl odpov.
   SIG  - Podpis zprvy - libovoln slo od 00H do FFH. Stejn slo, kter
          bylo poslno v dotazu, se vrt v odpovdi, m lze snadno rozpoznat,
          na kter dotaz odpov pila.
   INST - Kd instrukce - doporuen rozsahy instrukn sady jsou popsny ne
   ACK  - Potvrzen dotazu (Acknowledge), zda a jak byl proveden.
          ACK jsou z intervalu 00H a 0FH aby se odliily od kdu instrukce a
          maj nsledujc vznam:
           ($00) - ve v podku
           ($01) - jin chyba
           ($02) - neplatn kd instrukce
           ($03) - neplatn data
           ($04) - nepovolen zpis/pstup odmtnut
           ($05) - porucha zazen

           ($0C) - automaticky vyslan zprva  penos dat
           ($0D) - automaticky vyslan zprva - zmna log. hodnoty na
                                                univerzlnm vstupu
           ($0F) - automaticky vyslan zprva - ostatn
           ($0E) - automaticky vyslan zprva - kontinuln men
   DATA - Libovoln data, dle popisu instrukn sady.
   SUMA - Kontroln souet je souet vech bajt od prefixu po posledn bajt dat
          odeten od 255. Na instrukci s chybnm kontrolnm soutem nepijde
          odpov.
          SUMA = 255  (PRE + FRM + NUM + ADR + SIG + ACK (INST) + DATA)
   CR   - Zakonovan znak (0DH)

  Poznmky:
    1) Adresa FFH je rezervovna pro broadcast. Pokud je v dotazu adresa FFH,
       zazen se chov tak, jako by byla uvedena jeho adresa. Na dotazy
       s touto adresou se nevrac dn odpov.
       [const ADR_Universal]

    2) Adresa FEH je univerzln adresa. Pokud je v dotazu adresa FEH, zazen
       se chov tak, jako by byla uvedena jeho adresa. V odpovdi zazen uvede
       skutenou prv nastavenou adresu. Univerzln adresa se pouv jen v
       ppadech, kdy je na lince pipojen jen jedno zazen.
       [const ADR_Broadcast]

*)

unit uSpinel97;

interface

uses uSpinelCore;

const

  s97_FRM                  = #97;  // formt S97 protokolu SPINEL "a"


// Zvltn adresy formatu S97 - ADR ($FE-$FF)

  s97_ADR_Universal        = #$FE; // zazen kter obdr zprvu s touto
                                   // adresou se chovaj tak, jako by byla
                                   // uvedena jeho adresa. Ppadn odpov
                                   // se odesl.

  s97_ADR_Broadcast        = #$FF; // zazen kter obdr zprvu s touto
                                   // adresou se chovaj tak, jako by byla
                                   // uvedena jeho adresa. Ppadn odpov
                                   // se neodesl!


// Potvrzovac kdy - ACK ($00-$05)

  s97_ACK_OK               = $00; // VE V PODKU
                                  // Instrukce byla v podku pijata a kompletn
                                  // provedena

  s97_ACK_Error            = $01; // JIN CHYBA
                                  // Ble nespecifikovan chyba zazen

  s97_ACK_Invalid_INST     = $02; // NEPLATN KD INSTRUKCE
                                  // Pijat kd instrukce nen znm

  s97_ACK_Invalid_DATA     = $03; // NPLATN DATA
                                  // Data nemaj platnou dlku nebo obsahuj
                                  // neplatnou hodnotu.

  s97_ACK_Acces_denied     = $04; // NEPOVOLEN ZPIS/PSTUP ODMTNUT
                                  // Dotaz nebyl proveden, protoe nebyly splnny
                                  // urit podmnky.
                                  // - Pokus o zpis dat do nepstupn pamti.
                                  // - Snaha o aktivovn funkce zazen,
                                  //   kter vyaduje jin nastaven (nap.
                                  //   vy komunikan rychlost).
                                  // - Snaha o zmnu konfigurace, bez bezprostedn
                                  //   pedchzejcho povolen nastaven.
                                  // - Pstup do pamti chrnn heslem.

  s97_ACK_Device_failure   = $05; // PORUCHA ZAZEN
                                  // - Porucha zazen, vyadujc servisn zsah.
                                  // - Chyba vnitn pamti zazen nebo pamti
                                  //   nastaven.
                                  // - Chyba nkter vnitn periferie zazen
                                  //   (bhov chyba nebo chyba pi inicializaci).
                                  // - Jakkoli jin chyba ovlivujc sprvnou
                                  //   funkci zazen.

// Kdy ACK samovoln vyslanch zprv - ACK ($0C-$0F)

  s97_ACK_AutoTX_DATA      = $0C; // AUTOMATICKY VYSLAN INSTRUKCE  PENOS DAT
                                  // Odesln namench hodnot pi dosaen
                                  // kritick hranice sledovan veliiny.

  s97_ACK_AutoTX_InputChng = $0D; // AUTOMATICKY VYSLAN INSTRUKCE -
                                  // ZMNA LOG. HODNOTY NA UNIVERZLNM VSTUPU
                                  // - Hldn stav koncovch spna.

  s97_ACK_AutoTX_Measure   = $0E; // AUTOMATICKY VYSLAN INSTRUKCE  KONTINULN
                                  //                                 MEN
                                  // - Periodick odesln namench hodnot.

  s97_ACK_AutoTX_Other     = $0F; // AUTOMATICKY VYSLAN INSTRUKCE  OSTATN


// Rozsahy nvratovch kd ACK

  s97_set_ACK        =  [$00..$0F]; // Rozsah vech monch ACK

  s97_set_ACK_reply  =  [$00..$05]; // Rozsah vech nvratovch ACK kd
  s97_set_ACK_AutoTX =  [$0C..$0F]; // Rozsah vech ACK v samovoln vyslanch paketech

// Doporuen rozsahy instruknch kd INST

  s97_set_INST                 = [$10..$FF]; // Rozsah vech monch INST

  s97_set_INST_DeviceSet       = [$10..$1F]; // Nastaven zazen
  s97_set_INST_WDigitalOutputs = [$20..$2F]; // Nastaven dvoustavovch vstup a vstup
  s97_set_INST_RDigitalIO      = [$30..$3F]; // ten dvoustavovch vstup a vstup
  s97_set_INST_WAnalogOutputs  = [$40..$4F]; // Nastaven vstupnch spojitch veliin
  s97_set_INST_RAnalogIO       = [$50..$5F]; // ten vstupnch spojitch veliin
  s97_set_INST_Counters        = [$60..$7F]; // tae

  s97_set_INST_DataTx          = [$80..$8F]; // Penos dat  pjem
  s97_set_INST_DataRx          = [$90..$9F]; // Penos dat  vysln
  s97_set_INST_Other           = [$A0..$DF]; // Ostatn instrukce
  s97_set_INST_WSystemSettings = [$E0..$EF]; // Zpis do zazen  systmov nastaven
  s97_set_INST_RSystemSettings = [$F0..$FF]; // ten ze zazen  systmov nastaven

type

// Definice PAKETU protokolu SPINEL format S97

  TSpinelFormat97=record
   PRE: Char;                // prefix = vdy "*"
   FRM: Byte;                // formt
   NUM: Word;                // dlka paketu
   ADR: Char;                // adresa
   SIG: Byte;                // podpis
   INST_ACK: Byte;           // instrukce / potvrzen
   DATA: String;             // data
   SUMA: Byte;               // crc
   CR: Char;                 // cr = vdy #13 (enter)
  end;

  Tsf97=TspinelFormat97;     // Zkracen nzev typu pro PAKET protokolu SPINEL format S97

// ** Instrukce pro praci s protokolem SPINEL format S97 **

// sEncode_S97 - funkce ktere vytvori paket formatu S97 vysledny string 
//               staci poslat na komunikacni kanal
function sEncode_S97(ADR:Char; SIG:byte; INST:byte; DATA:String=''):String; overload;
function sEncode_S97(SIG:byte; INST:byte; DATA:String=''):String; overload;
function sEncode_S97(INST:byte; DATA:String=''):String; overload;
function sEncode_S97Broadcast(SIG:byte; INST:byte; DATA:String=''):String;

// sDecode_97 - funkce dekoduje spinelovsky paket formatu S97 a vysledek preda 
//              jako strukturu obsahujici vsechny casti paketu
function sDecode_S97(PACKET:String):TSpinelFormat97;

// sGetPacket_97 - pokusi se najit nejblizsi paket formatu S97 a vrati jej, 
//                 pokud je nastavena promenna DelData odstrani data od zacatku 
//                 az po konec paketu se zdrojoveho stringu
function sGetPacket_S97(var Data:String; DelData:Boolean=false):string;

function sChecek_S97(PACKET:String):Word;

function s97_getADR(PACKET:String; CheckPacket:boolean=False):Byte;
function s97_getADRch(PACKET:String; CheckPacket:boolean=False):Char;

implementation

uses SysUtils, uSpinel;

function sEncode_S97(ADR:Char; SIG:byte; INST:byte; DATA:String):string;
var NUM:Word;
begin
  NUM:=(3+Length(DATA)+2);
  result:=PRE+s97_FRM+Char(NUM div $100)+Char(NUM mod $100)+ADR+Char(SIG)+Char(INST)+DATA;
  result:=Result+char(sCountCRC(Result))+CR;
end;

function sEncode_S97(SIG:byte; INST:byte; DATA:String):string; overload;
begin
  result:=sEncode_S97(s97_ADR_Universal,SIG,INST,DATA);
end;

function sEncode_S97(INST:byte; DATA:String=''):String; overload;
begin
  result:=sEncode_S97(s97_ADR_Universal,$FF,INST,DATA);
end;

function sEncode_S97Broadcast(SIG:byte; INST:byte; DATA:String):string;
begin
  result:=sEncode_S97(s97_ADR_Broadcast,SIG,INST,DATA);
end;

function sDecode_S97(PACKET:String):TSpinelFormat97;
begin
  result.CR:=#0;
  try
    If Length(PACKET)<9 then raise Exception.Create('');
    result.PRE:=PACKET[1];
    result.FRM:=Ord(PACKET[2]);
    result.NUM:=Ord(PACKET[3])*$100+Ord(PACKET[4]);
    result.ADR:=PACKET[5];
    result.SIG:=Ord(PACKET[6]);
    result.INST_ACK:=Ord(PACKET[7]);
    result.DATA:=Copy(PACKET,8,Result.NUM-5);
    result.SUMA:=Ord(PACKET[4+Result.NUM-1]);
    result.CR:=PACKET[4+Result.NUM];
  except
    raise Exception.Create('Incorrect PACKET (Spinel format 97)!');
  end;
end;

function sGetPacket_S97(var Data:String; DelData:Boolean=false):string;
(*
 Pokud je na potku etezce "Data" platn paket ve formatu spinel S97,
 tj. odpovd dlka, prefix a sufix, bude vsledkem funkce pekoprovan
 st etzce "Data! s tmto paketem, jinak vrac przdn etzec.
   Delete - (true) = Odmae se ze vstupnho bufferu "Data" nalezen paket.
*)
var NUM: Word;
begin
  result:='';
  try
    if Length(Data)<4 then Exit;
    if (Data[1]<>PRE) or (Data[2]<>s97_FRM) then Exit;

    NUM:=Ord(Data[3])*$100+Ord(Data[4]);
    if Length(Data)<NUM+4 then Exit;

    Result:=Copy(Data,1,NUM+4);

    if DelData then Delete(Data,1,Length(result));
  except
  end;
end;

function sChecek_S97(PACKET:String):Word;
(*
 Chybov kdy funkce sCheck97
  0 - Ok
  1 - Pli krtk paket
  2 - Neplatn prefix paketu (PRE)
  3 - Paket m jin pznak formatu (FRM)
  4 - Chybn dlka paketu (NUM)
  5 - Chybn kontroln souet (CRC)
  6 - Neplatn koncov znak (CR)
*)
begin
  result:=sACK_Ok;
  if Length(PACKET)<8 then result:=1 else
  if PACKET[1]<>PRE then result:=2 else
  if PACKET[2]<>s97_FRM then result:=3 else
  if Ord(PACKET[3])*$100+Ord(PACKET[4])<>Length(PACKET)-4 then result:=4 else
  if sCountCRC(Copy(PACKET,1,Length(PACKET)-2))<>Ord(PACKET[Length(PACKET)-1]) then result:=5 else
  if PACKET[Length(PACKET)]<>CR then result:=6;
end;

{
function sPacketDirection_S97(PACKET:String):TsPacketDirection;
begin
  result:=spd_Unknown;
  if sChecek_S97(PACKET)=0 then
   if Ord(PACKET[7])<$10
    then result:=spd_INST
    else result:=spd_ACK;
end;
}

function s97_getADRch(PACKET:String; CheckPacket:boolean=False):Char;
var W:Word;
begin
  W:=0;

  if CheckPacket
   then W:=sChecek_S97(PACKET)
   else W:=ord(NOT((Length(PACKET)>4) and (PACKET[1]=PRE) and (PACKET[2]=s97_FRM)));

  if W=0
   then result:=PACKET[5]
   else raise Exception.Create('INCORRECT SPINEL PACKET FORMAT S97');
end;

function s97_getADR(PACKET:String; CheckPacket:boolean=False):Byte; overload;
begin
  result:=Ord(s97_getADRch(PACKET,CheckPacket));
end;


end.
